@@ -21,7 +21,10 @@
 
 #include <stdlib.h>
 
-#include "error.h"
+#ifndef GNULIB_LIBPOSIX
+# include "error.h"
+#endif
+
 #include "exitfail.h"
 
 #include "gettext.h"
@@ -30,10 +33,11 @@
 void
 openat_save_fail (int errnum)
 {
+#ifndef GNULIB_LIBPOSIX
   error (exit_failure, errnum,
          _("unable to record current working directory"));
-
-  /* The `noreturn' attribute cannot be applied to error, since it returns
+#endif
+  /* _Noreturn cannot be applied to error, since it returns
      when its first argument is 0.  To help compilers understand that this
      function does not return, call abort.  Also, the abort is a
      safety feature if exit_failure is 0 (which shouldn't happen).  */
@@ -48,8 +52,10 @@ openat_save_fail (int errnum)
 void
 openat_restore_fail (int errnum)
 {
+#ifndef GNULIB_LIBPOSIX
   error (exit_failure, errnum,
          _("failed to return to initial working directory"));
+#endif
 
   /* As above.  */
   abort ();
