@@ -1,6 +1,6 @@
 /* Compare numeric strings.
 
-   Copyright (C) 2005, 2006, 2009 Free Software Foundation, Inc.
+   Copyright (C) 2005-2006, 2009-2011 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -23,7 +23,7 @@
 
 /* Externally-visible name for numcompare.  */
 
-int
+int _GL_ATTRIBUTE_PURE
 strnumcmp (char const *a, char const *b,
            int decimal_point, int thousands_sep)
 {
