@@ -102,17 +102,45 @@ uid_to_name (uid_t uid)
                   : umaxtostr (uid, buf));
 }
 
+/* Allocate a string representing USER and GROUP.  */
+
+static char *
+user_group_str (char const *user, char const *group)
+{
+  char *spec = NULL;
+
+  if (user)
+    {
+      if (group)
+        {
+          spec = xmalloc (strlen (user) + 1 + strlen (group) + 1);
+          stpcpy (stpcpy (stpcpy (spec, user), ":"), group);
+        }
+      else
+        {
+          spec = xstrdup (user);
+        }
+    }
+  else if (group)
+    {
+      spec = xstrdup (group);
+    }
+
+  return spec;
+}
+
 /* Tell the user how/if the user and group of FILE have been changed.
    If USER is NULL, give the group-oriented messages.
    CHANGED describes what (if anything) has happened. */
 
 static void
 describe_change (const char *file, enum Change_status changed,
+                 char const *old_user, char const *old_group,
                  char const *user, char const *group)
 {
   const char *fmt;
-  char const *spec;
-  char *spec_allocated = NULL;
+  char *old_spec;
+  char *spec;
 
   if (changed == CH_NOT_APPLIED)
     {
@@ -121,35 +149,32 @@ describe_change (const char *file, enum Change_status changed,
       return;
     }
 
-  if (user)
-    {
-      if (group)
-        {
-          spec_allocated = xmalloc (strlen (user) + 1 + strlen (group) + 1);
-          stpcpy (stpcpy (stpcpy (spec_allocated, user), ":"), group);
-          spec = spec_allocated;
-        }
-      else
-        {
-          spec = user;
-        }
-    }
-  else
-    {
-      spec = group;
-    }
+  spec = user_group_str (user, group);
+  old_spec = user_group_str (user ? old_user : NULL, group ? old_group : NULL);
 
   switch (changed)
     {
     case CH_SUCCEEDED:
-      fmt = (user ? _("changed ownership of %s to %s\n")
-             : group ? _("changed group of %s to %s\n")
+      fmt = (user ? _("changed ownership of %s from %s to %s\n")
+             : group ? _("changed group of %s from %s to %s\n")
              : _("no change to ownership of %s\n"));
       break;
     case CH_FAILED:
-      fmt = (user ? _("failed to change ownership of %s to %s\n")
-             : group ? _("failed to change group of %s to %s\n")
-             : _("failed to change ownership of %s\n"));
+      if (old_spec)
+        {
+          fmt = (user ? _("failed to change ownership of %s from %s to %s\n")
+                 : group ? _("failed to change group of %s from %s to %s\n")
+                 : _("failed to change ownership of %s\n"));
+        }
+      else
+        {
+          fmt = (user ? _("failed to change ownership of %s to %s\n")
+                 : group ? _("failed to change group of %s to %s\n")
+                 : _("failed to change ownership of %s\n"));
+          free (old_spec);
+          old_spec = spec;
+          spec = NULL;
+        }
       break;
     case CH_NO_CHANGE_REQUESTED:
       fmt = (user ? _("ownership of %s retained as %s\n")
@@ -160,9 +185,10 @@ describe_change (const char *file, enum Change_status changed,
       abort ();
     }
 
-  printf (fmt, quote (file), spec);
+  printf (fmt, quote (file), old_spec, spec);
 
-  free (spec_allocated);
+  free (old_spec);
+  free (spec);
 }
 
 /* Change the owner and/or group of the FILE to UID and/or GID (safely)
@@ -230,12 +256,10 @@ restricted_chown (int cwd_fd, char const *file,
         }
     }
 
-  { /* FIXME: remove these curly braces when we assume C99.  */
-    int saved_errno = errno;
-    close (fd);
-    errno = saved_errno;
-    return status;
-  }
+  int saved_errno = errno;
+  close (fd);
+  errno = saved_errno;
+  return status;
 }
 
 /* Change the owner and/or group of the file specified by FTS and ENT
@@ -459,8 +483,13 @@ change_file_owner (FTS *fts, FTSENT *ent,
              : !symlink_changed ? CH_NOT_APPLIED
              : !changed ? CH_NO_CHANGE_REQUESTED
              : CH_SUCCEEDED);
+          char *old_usr = file_stats ? uid_to_name (file_stats->st_uid) : NULL;
+          char *old_grp = file_stats ? gid_to_name (file_stats->st_gid) : NULL;
           describe_change (file_full_name, ch_status,
+                           old_usr, old_grp,
                            chopt->user_name, chopt->group_name);
+          free (old_usr);
+          free (old_grp);
         }
     }
 
